;********************************************************************
;*																	*
;*   Мини ОС реального времени										*
;*   http://easyelectronics.ru										*
;*																	*
;*   Кристал: ATTiny13                                              *
;*																	*
;*	Автоматика управления дворниками для ВАЗ21213					*
;*																	*
;*		define.inc	- Константы и дифайны							*
;*																	*
;*   Структура проекта:												*
;*		main.asm	- главный модуль (основной цикл)				*
;*		interupts.asm - Модуль прерываний							*
;*		tasks.asm 	- Модуль процессов								*
;* 		taskq.asm	- Модуль очереди процессов						*
;*		timers.asm	- Модуль очереди таймеров						*
;*		data.inc	- Данные в программной памяти					*
;*		define.inc	- Константы и дифайны							*
;*																	*
;********************************************************************

;------Working registers-----------------
.def	buffer	=	r0		;буферный регистр
.def	btnTimerFlag = r1	;флаг работы таймера кнопки
.def	btnCnt	=	r2		;счетчик количества коротких нажатий
.def	btnFlag	=	r3		;флаг предидущего состояния кнопки (борьба с дребезгом)
.def	btnLDawn=	r4		;флаг начала и окончания долгого нажатия
.def	btnSt	=	r5		;состояние кнопки при входе в процедуру сканирования
.def 	return	=	r6		;значение возврата из процедуры
.def	circleFlag	=	r7	;флаг рабочего режима дворников
.def	P0WpMOn	=	r8		;флаг работы дворников в режиме P_0
.def	zero	=	r14		;Нулевой регистр
.def	one		= 	r15		;Регистр со значением 1
.def	OSWR	=	r16		;Рабочий регистр ОС
.def	tmp1	=	r17		;Регистр хранения временных данных
.def	tmp2	=   r18		;Регистр хранения временных данных
.def	counter	=	r19		;Регистр счетчик


;------Constants-------------------------
.equ	TaskQuereSize	=	5	;Размер очереди задач
.equ	TimersQuereSize	=	8	;Размер очереди таймеров
.equ	QuereSize = TaskQuereSize + TimersQuereSize * 3 ;общая длина очереди
;------коды задач-------------------------
.equ	TS_idle		=	0		;Код задачи 0 - простой ядра
.equ	TS_Proces	=	1		;Код задачи обработки результатов опроса кнопки и рычага
.equ	TS_ScanBtn	=	2		;Код задачи сканирования кнопки
.equ	TS_ScanSwitch	=	3	;Код задачи сканирования положения рычага
.equ	TS_WpM_On	=	4		;Код задачи включения мотора дворников
.equ	TS_WpM_Off	=	5		;Код задачи выключения мотора дворников
.equ	TS_WtM_On	=	6		;Код задачи включения мотора омывателя
.equ	TS_WtM_Off	=	7		;Код задачи выключения мотора омывателя
.equ	TS_ClrCF	=	8		;Код задачи сброса флага цикла работы дворников
;------настройка порта B-------------------
.equ	ST_DDR		=	0b00011000 ;PB0-PB2 на вход, PB3,PB4 на выход
.equ	ST_PORTB	=	0b00000011 ;подтягивающие регистры на PB0,PB1, PB2 весит в воздухе
;------биты порта B------------------------
.equ	BtnBit		= 	0	;вход кнопки
.equ	SwitchBit	=	1	;вход рычага
.equ	CntrlBit	= 	2	;контрольный вход
.equ	WpMBit		= 	3	;выход управления мотора дворников
.equ	WtMBit		= 	4	;выход управления мотора омывателя
;------маски включения и отключения моторов-
.equ	WpMOnMask	= 	0b00001000	;маска для включения мотора дворников
.equ	WpMOffMask	=	0b11110111	;маска для отключения мотора дворников
.equ	WtMOnMask	=	0b00010000	;маска для включения мотора омывателя
.equ	WtMOffMask	=   0b11101111	;маска для отключения мотора дворников
;------состояние кнопки и рычага------------
.equ	UP		= 0		;кнопка отпущена после нажатия
.equ	DN		= 1		;кнопка нажата после отжатия
.equ	LUP		= 2		;кнопка давно отжата
.equ	LDN		= 3		;кнопка давно нажата
.equ	P_0		= 0		;положения рычага 0 - дворники отключены
.equ	P_1		= 1		;положение рычага 1 - прерывистый режим работы дворников
.equ	P_2		= 2		;положение рычага 2 - постояная работа дворников
;------временые интервалы в мс--------------
.equ	SS_Time	= 50		;Период сканирования положения рычага дворников
.equ	SB_Time = 3			;Период сканирования кнопки
.equ	PS_Time = 5			;Период обработки состояния кнопки и рычага
.equ	ClickOver = 500		;Время после которого нажатие или отпускание считается долгим
.equ	ManualWiperOn = 3500	;Время паузы старта мотора дворников при ручном омывании 
.equ	ManualWiperOff= 3500	;Время паузы останова мотора дворников при ручном омывании
.equ	OneWiperTime  = 1000	;Время подачи питания на мотор дворников при прерывистом режиме работы
.equ	PauseMax	  = 7000	;Максимальное время паузы при прерывистом режиме работы
.equ	PauseMin	  = 3000	;Минимальное время паузы при прерывистом режиме работы
.equ	Delta = (PauseMax-PauseMin) / 2 ;Время уменьшения паузы
.equ	AutoWaterTime  = 6000	;Время работы мотора омывателя в автоматическом режиме
.equ	AutoWiperTimeOn=3500	;Время паузы включения мотора дворников в автоматическом режиме
.equ	AutoWiperTimeOff=10000	;Время отключения мотора дворников в автоматическом режиме
.equ	AutoClrCicleFlagOff=10500 ;Время сброса флага цикла работы дворников в автоматическом режиме
