;********************************************************************
;*																	*
;*   Мини ОС реального времени										*
;*   http://easyelectronics.ru										*
;*																	*
;*   Кристал: ATTiny13                                              *
;*																	*
;*	Автоматика управления дворниками для ВАЗ21213					*
;*																	*
;*   taskq.asm	- Модуль очереди процессов							*
;*																	*
;*   Структура проекта:												*
;*		main.asm	- главный модуль (основной цикл)				*
;*		tasks.asm 	- Модуль процессов								*
;* 		taskq.asm	- Модуль очереди процессов						*
;*		timers.asm	- Модуль очереди таймеров						*
;*		data.inc	- Данные в программной памяти					*
;*		define.inc	- Константы и дифайны							*
;*																	*
;********************************************************************

;==== Инициализация очереди задач ===================================
InitTaskQuere:
	;Заполняем всю очередь 0xFF - нет задач
	ldi ZL, Low(TaskQuere)		;В Z начало очереди задач
	ldi ZH, High(TaskQuere)
	ldi tmp1, 0xFF
	ldi counter, TaskQuereSize	;Длина очереди задач
ITQ_L01:
	st Z+, tmp1
	dec counter
	brne ITQ_L01
	ret

;==== Постановка задачи в очередь ===================================
SetTask:
	;Номер задачи передается через OSWR
	;Сохраняем в стеке используемые регистры
	push ZH
	push ZL
	push tmp1
	in tmp1, SREG
	push tmp1
	push counter
	
	ldi ZL, Low(TaskQuere)		;В Z начало очереди задач
	ldi ZH, High(TaskQuere)
	ldi counter, TaskQuereSize	;Длина очереди задач
	cli							;Запрещаем прерывания
STQ_L01:
	ld tmp1, Z+
	cpi tmp1, 0xFF 	;Ищем первый свободный слот
	breq STQ_L02	;в очереди задач
	dec counter
	brne STQ_L01
	rjmp STQ_L03	;Не нашли очередь полная((( задача в очередь не ставиться
STQ_L02:
	st -Z, OSWR		;устанавливаем номер задачи в очередь
STQ_L03:
	;Востонавливаем из стека рабочие регистры
	pop counter
	pop tmp1
	out SREG, tmp1
	pop tmp1
	pop ZL
	pop ZH
	ret

;==== Обработка очереди задач =======================================
ProcesTaskQuere:
	ldi ZL, Low(TaskQuere)		;В Z начало очереди задач
	ldi ZH, High(TaskQuere)
	ld	OSWR, Z					;Читаем первый номер задачи в очереди
	cpi OSWR, 0xFF
	breq PTQ_L02				;Если очередь пуста выходим
	;Вычесляем адрес процесса
	clr ZH
	lsl OSWR					;Умножаем номер очереди на два
	mov ZL, OSWR
	subi ZL, Low(-TaskProcs*2)	;Складываем с началом таблицы задач
	sbci ZH, High(-TaskProcs*2)
	lpm							;Достаем адрес задачи из таблицы
	mov OSWR, R0				;В OSWR младший байт адреса
	ld R0, Z+					;Увеличиваем Z на один
	lpm							;В R0 старший байт адреса
	push R0						;Сохраняем его в стеке
	push OSWR
	;Продвинем очередь
	ldi counter, TaskQuereSize - 1
	ldi ZL, Low(TaskQuere)		;В Z начало очереди задач
	ldi ZH, High(TaskQuere)
	cli							;Запрещаем прерывания
PTQ_L01:
	ldd OSWR, Z+1				;Перекладываем номер задачи из Z+1
	st 	Z+, OSWR				;В Z
	dec counter
	brne PTQ_L01
	ldi OSWR, 0xFF				;В последний слот очереди признак
	st	Z+, OSWR				;конца очереди - 0xFF
	sei							;Разрешаем прерывания
	pop ZL						;Достаем адрес задачи
	pop ZH
	ijmp						;и переходим на задачу
PTQ_L02:	
	ret
